use super::{NPCPronounType, NPCSayInfo, NPCSayType, NPCSpawnPossession, NPC};
use crate::{
    models::{
        consent::ConsentType,
        item::{LocationActionType, Pronouns, SkillType},
    },
    static_content::{
        npc::{npc_pronoun_type_to_pronouns, KillBonus},
        possession_type::PossessionType,
    },
};
use serde::Deserialize;
use serde_yaml::from_str as from_yaml_str;

#[derive(Deserialize)]
enum SewerNPC {
    RadiantPredator {
        code: String,
        name: String,
        spawn_loc: String,
        gender: NPCPronounType,
    },
}

pub fn npc_list() -> Vec<NPC> {
    use NPCSayType::FromFixedList;
    let radiant_predator_stdsay = NPCSayInfo {
        say_code: "babble",
        frequency_secs: 120,
        talk_type: FromFixedList(vec![
            "I hunger for flesh.",
            "The darkness is my ally.",
            "Step closer please, prey.",
            "My stomach rumbles for human flesh.",
            "Survival is a predator's only law.",
            "The scent of fear is intoxicating.",
            "I am the shadow in the sewer.",
            "In the dark, I thrive.",
            "Flesh is my sustenance.",
            "My predator knife is fantastic for hunting unsuspecting prey.",
            "In the depths, I reign supreme.",
            "The hunt begins.",
            "No mercy for the intruders.",
            "I love visitors to the sewers, they taste great!",
            "The echoes reveal your presence.",
            "My hunger knows no bounds.",
            "The predator way is to be silent, deadly, and relentless.",
            "The sewer is my kingdom.",
            "Prey that comes my way doesn't get away.",
            "Do I smell a human... in my sewer? Yum.",
            "I know you are there, prey! Give up, we might as well get this over with!",
        ]),
    };

    from_yaml_str::<Vec<SewerNPC>>(include_str!("sewer_npcs.yaml"))
        .unwrap()
        .into_iter()
        .map(|c| match c {
            SewerNPC::RadiantPredator { code, name, spawn_loc, gender } =>
              NPC {
                  code: format!("sewer_{}", &code),
                  name,
                  pronouns: Pronouns {
                      is_proper: false,
                      ..npc_pronoun_type_to_pronouns(&gender)
                  },
                  aliases: vec!["predator".to_owned(), "radiant".to_owned(),
                                "radiant predator".to_owned()],
                  aggression: 12,
                  description: "A creature, vicious in demeanour, but uncannily human like, but also a bit... off, in a grotesquely mutant way. Discoloured flesh hangs loosely around the disproportionate frame, while a horrific grin reveals phosphorescent yellowish teeth, sharper than those of a normal human. You catch a whiff of a putrid odour, and notice fragments of meat caught between the teeth of this hideous humanoid. Coarse brown matted hair over the body gives it an ape-like appearance. The eyes are larger than those of a normal human and gleam, leading you to suspect this creature is well adapted to low light levels".to_owned(),
                  spawn_location: format!("room/melbs_sewers_{}", &spawn_loc),
                  spawn_possessions: vec![
                      NPCSpawnPossession {
                          what: PossessionType::Dagger,
                          action_type: LocationActionType::Wielded,
                          wear_layer: 0,
                      },
                  ],
                  kill_bonus: Some(KillBonus {
                      msg: "On your wristpad: You are doing a great job removing hazards from the sewer! Let me toss a few credits your way.",
                      payment: 200,
                  }),
                  aggro_pc_only: true,
                  total_xp: 10000,
                  total_skills: SkillType::values()
                      .into_iter()
                      .map(|sk| {
                          (
                              sk.clone(),
                              match sk {
                                  SkillType::Dodge => 14.0,
                                  SkillType::Blades => 18.0,
                                  _ => 8.0
                              }
                          )
                      }).collect(),
                  player_consents: vec!(ConsentType::Fight),
                  message_handler: None,
                  wander_zones: vec!["melbs_sewers".to_owned()],
                  says: vec!(radiant_predator_stdsay.clone()),
                  ..Default::default()
              }
        }).collect()
}
